// 
// Copyright (C) 2015, NinjaTrader LLC <www.ninjatrader.com>.
// NinjaTrader reserves the right to modify or overwrite this NinjaScript component with each release.
//
#region Using declarations
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.ComponentModel.DataAnnotations;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Xml.Serialization;
using NinjaTrader.Cbi;
using NinjaTrader.Gui;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.SuperDom;
using NinjaTrader.Data;
using NinjaTrader.NinjaScript;
using NinjaTrader.Core.FloatingPoint;
using NinjaTrader.NinjaScript.DrawingTools;
#endregion

// This namespace holds indicators in this folder and is required. Do not change it.
namespace NinjaTrader.NinjaScript.Indicators.Sim22
{
	/// <summary>
	/// Port of 'Reverse Engineering RSI' by Giorgos Siligardos TASC June 2003. Sim22 Feb 2016. NT8b8.
	/// </summary>
	public class Sim22_ReverseEngineerRSI : Indicator
	{
		private Series<double>		avgDown;
		private Series<double>		avgUp;
		
		private double				constant3;
		private Series<double>		down;
		private SMA					smaDown;
		private	SMA					smaUp;
		private Series<double>		up;

		protected override void OnStateChange()
		{
			if (State == State.SetDefaults)
			{
				Description					= @"Port of 'Reverse Engineering RSI' by Giorgos Siligardos TASC June 2003. Sim22 Feb2016. NT8b8.";
				Name						= "ReverseEngineerRSISim22";
				IsSuspendedWhileInactive	= true;
				IsOverlay					= true;
				Period						= 14;
				rsiValue					= 50.00;
				Displacement				= 1;
				
				AddPlot(Brushes.Blue, "RevEngRSI");	
			}
			else if (State == State.Configure)
			{
				avgUp				= new Series<double>(this);
				avgDown				= new Series<double>(this);
				down				= new Series<double>(this);
				up					= new Series<double>(this);
				smaDown				= SMA(down, Period);
				smaUp				= SMA(up, Period);

				constant3 = (Period - 1);	
			}
		}

		public override string DisplayName
		{
		    get { return "RevEngRSI (" + Period + ") Level = " + rsiValue; }
		}
		
		protected override void OnBarUpdate()
		{
			if (CurrentBar == 0)
			{
				down[0]		= 0;
				up[0]		= 0;
				return;
			}

			double input0	= Input[0];
			double input1	= Input[1];
			down[0]			= Math.Max(input1 - input0, 0);
			up[0]			= Math.Max(input0 - input1, 0);

			if (CurrentBar + 1 < Period) 
			{	
				return;
			}

			if ((CurrentBar + 1) == Period) 
			{
				// First averages 
				avgDown[0]	= smaDown[0];
				avgUp[0]	= smaUp[0];
			}  
			else 
			{
				// Rest of averages are smoothed
				avgDown[0]	= (avgDown[1] * constant3 + down[0]) / Period;
				avgUp[0]	= (avgUp[1] * constant3 + up[0]) / Period;
			}

			// You can add extra plots using this calculation
			RevEngRSI[0]		= CalculateRER(rsiValue);
		}
		
		// You can add extra plots using this calculation
		protected double CalculateRER(double level)
		{
			double X			= (Period - 1) * (avgDown[0] * level/(100 - level) - avgUp[0]);
			double rer			= X >= 0 ? Input[0] + X : Input[0] + X *(100 - level)/ level;	
			return rer;
		}

		#region Properties
		

		[Browsable(false)]
		[XmlIgnore()]
		public Series<double> RevEngRSI
		{
			get { return Values[0]; }
		}
		
		
		[Range(1, int.MaxValue), NinjaScriptProperty]
		[Display(ResourceType = typeof(Custom.Resource), Name = "Period", GroupName = "NinjaScriptParameters", Order = 0)]
		public int Period 
		{ get; set; }
		
		[Range(1, 100), NinjaScriptProperty]
		[Display(ResourceType = typeof(Custom.Resource), Name = "RSI value", GroupName = "NinjaScriptParameters", Order = 0)]
		public double rsiValue
		{ get; set; }
		
		#endregion
	}
}

#region NinjaScript generated code. Neither change nor remove.

namespace NinjaTrader.NinjaScript.Indicators
{
	public partial class Indicator : NinjaTrader.Gui.NinjaScript.IndicatorRenderBase
	{
		private Sim22.Sim22_ReverseEngineerRSI[] cacheSim22_ReverseEngineerRSI;
		public Sim22.Sim22_ReverseEngineerRSI Sim22_ReverseEngineerRSI(int period, double rsiValue)
		{
			return Sim22_ReverseEngineerRSI(Input, period, rsiValue);
		}

		public Sim22.Sim22_ReverseEngineerRSI Sim22_ReverseEngineerRSI(ISeries<double> input, int period, double rsiValue)
		{
			if (cacheSim22_ReverseEngineerRSI != null)
				for (int idx = 0; idx < cacheSim22_ReverseEngineerRSI.Length; idx++)
					if (cacheSim22_ReverseEngineerRSI[idx] != null && cacheSim22_ReverseEngineerRSI[idx].Period == period && cacheSim22_ReverseEngineerRSI[idx].rsiValue == rsiValue && cacheSim22_ReverseEngineerRSI[idx].EqualsInput(input))
						return cacheSim22_ReverseEngineerRSI[idx];
			return CacheIndicator<Sim22.Sim22_ReverseEngineerRSI>(new Sim22.Sim22_ReverseEngineerRSI(){ Period = period, rsiValue = rsiValue }, input, ref cacheSim22_ReverseEngineerRSI);
		}
	}
}

namespace NinjaTrader.NinjaScript.MarketAnalyzerColumns
{
	public partial class MarketAnalyzerColumn : MarketAnalyzerColumnBase
	{
		public Indicators.Sim22.Sim22_ReverseEngineerRSI Sim22_ReverseEngineerRSI(int period, double rsiValue)
		{
			return indicator.Sim22_ReverseEngineerRSI(Input, period, rsiValue);
		}

		public Indicators.Sim22.Sim22_ReverseEngineerRSI Sim22_ReverseEngineerRSI(ISeries<double> input , int period, double rsiValue)
		{
			return indicator.Sim22_ReverseEngineerRSI(input, period, rsiValue);
		}
	}
}

namespace NinjaTrader.NinjaScript.Strategies
{
	public partial class Strategy : NinjaTrader.Gui.NinjaScript.StrategyRenderBase
	{
		public Indicators.Sim22.Sim22_ReverseEngineerRSI Sim22_ReverseEngineerRSI(int period, double rsiValue)
		{
			return indicator.Sim22_ReverseEngineerRSI(Input, period, rsiValue);
		}

		public Indicators.Sim22.Sim22_ReverseEngineerRSI Sim22_ReverseEngineerRSI(ISeries<double> input , int period, double rsiValue)
		{
			return indicator.Sim22_ReverseEngineerRSI(input, period, rsiValue);
		}
	}
}

#endregion
